from datetime import datetime
from typing import Any

from bson import ObjectId
from bson.errors import InvalidId
from marshmallow import ValidationError, fields


class MarshamallowObjectId(fields.Field):
    def _serialize(self, value: ObjectId, attr: str | None, obj: Any, **kwargs):
        return str(value)

    def _deserialize(
        self, value: str | ObjectId | bytes, attr: str | None, data: Any, **kwargs
    ):
        try:
            return ObjectId(value)
        except (InvalidId, TypeError):
            raise ValidationError("Invalid ObjectId.")


class MarshamallowDateTime(fields.DateTime):
    def _deserialize(
        self, value: str | datetime, attr: str | None, data: Any, **kwargs
    ):
        if isinstance(value, datetime):
            return value
        return super()._deserialize(value, attr, data, **kwargs)



from .client import (
    DeleteMany,
    DeleteOne,
    InsertOne,
    ReplaceOne,
    UpdateMany,
    UpdateOne,
    initial_collections,
)
from .fields import (
    BooleanField,
    DateTimeField,
    DecimalField,
    DictField,
    EmbeddedField,
    FieldParamters,
    FloatField,
    IntegerField,
    ListField,
    ObjectIdField,
    StringField,
)
from .table import Index, Table

__all__ = [
    "DeleteMany",
    "DeleteOne",
    "InsertOne",
    "ReplaceOne",
    "UpdateMany",
    "UpdateOne",
    "initial_collections",
    "Index",
    "Table",
    "BooleanField",
    "DateTimeField",
    "DecimalField",
    "DictField",
    "EmbeddedField",
    "FieldParamters",
    "FloatField",
    "IntegerField",
    "ListField",
    "ObjectIdField",
    "StringField",
]



import dataclasses

import pytest

from typedmongo.expressions import (
    CombineExpression,
    CompareExpression,
    CompareMixin,
    NotExpression,
)


@dataclasses.dataclass(eq=False)
class Field(CompareMixin):
    field_name: str


field = Field("name")


@pytest.mark.parametrize(
    "expression, expected",
    [
        (field >= 18, CompareExpression(field, ">=", 18)),
        (field > 18, CompareExpression(field, ">", 18)),
        (field < 18, CompareExpression(field, "<", 18)),
        (field <= 18, CompareExpression(field, "<=", 18)),
        (field == "Aber", CompareExpression(field, "==", "Aber")),
        (field != "Aber", CompareExpression(field, "!=", "Aber")),
        (~(field == "Aber"), NotExpression(CompareExpression(field, "==", "Aber"))),
        (
            (field > 18) & (field < 35),
            CombineExpression(
                "AND",
                CompareExpression(field, ">", 18),
                CompareExpression(field, "<", 35),
            ),
        ),
        (
            (field > 18) | (field < 35),
            CombineExpression(
                "OR",
                CompareExpression(field, ">", 18),
                CompareExpression(field, "<", 35),
            ),
        ),
        (
            (field > 18) & (field < 35) | (field == 35),
            CombineExpression(
                "OR",
                CombineExpression(
                    "AND",
                    CompareExpression(field, ">", 18),
                    CompareExpression(field, "<", 35),
                ),
                CompareExpression(field, "==", 35),
            ),
        ),
        (
            ~((field > 18) & (field < 35)),
            CombineExpression(
                "OR",
                NotExpression(CompareExpression(field, ">", 18)),
                NotExpression(CompareExpression(field, "<", 35)),
            ),
        ),
        (
            ~((field > 18) | (field < 35)),
            CombineExpression(
                "AND",
                NotExpression(CompareExpression(field, ">", 18)),
                NotExpression(CompareExpression(field, "<", 35)),
            ),
        ),
        (
            ~((field > 18) & (field < 35) | (field == 35)),
            CombineExpression(
                "AND",
                CombineExpression(
                    "OR",
                    NotExpression(CompareExpression(field, ">", 18)),
                    NotExpression(CompareExpression(field, "<", 35)),
                ),
                NotExpression(CompareExpression(field, "==", 35)),
            ),
        ),
        (
            ~~((field > 18) & (field < 35)),
            CombineExpression(
                "AND",
                CompareExpression(field, ">", 18),
                CompareExpression(field, "<", 35),
            ),
        ),
    ],
)
def test_compile_expressions(expression, expected):
    assert expression == expected
